
'This version of SMALBOOT uses E2BOOT.BIN to boot up the hc11.
'
' The CONFIG data is derived mathematically by E2BOOT.BIN and is not
' transmitted to the HC11 separately.
'
'
' For 'A1 type chips, use A1BOOT.EXE which uses A1BOOT.BIN (no mode switch).
'
' Differences from SMALBOO7.BAS:
'
'
'  -Allows user programs to be as large as the entire EEPROM of the E2 chip.
'   In this case, user program must have vector list at end and be 2048 bytes.
'   A default pseudovector list is installed at the top of the EEPROM before
'   The program is loaded; this may be overwritten with a vector list that is
'   part of the user program if desied. In either case, chip is put in single
'   chip mode and user program is run after loading.
'
'  -Opens the serial port with all hardware handshake signals ignored, to
'   allow the use of a 3 wire line to the target system with no loopbacks
'   on the handshake lines. This improves compatibility with other software
'   used with Carson Electronics controllers and simplifies cable design.
'
'  -Uses the reception of a character to trigger bootloads in addition to
'   framing error detection, for compatibility with a USB to serial
'   adapter under Windows XP. This situation does not seem to pass framing
'   errors back through to this program. Recovery from an unwanted reset
'   during a bootloading session must be done manually in this case.
'

DECLARE SUB CONVtoASC ()
DECLARE SUB SCRNFORMAT ()
DECLARE SUB SENDBIN ()
DECLARE SUB ECHOBACK ()
DECLARE SUB QUIT ()

DIM SHARED COLUMN AS INTEGER
DIM SHARED ECHO   AS INTEGER
DIM SHARED MEMLOC AS LONG
DIM SHARED H$
DIM SHARED LCOM$
DIM SHARED SLIN$
DIM SHARED ASCII$
DIM SHARED OOPS$
DIM OOPS$(76)
DIM SHARED BB$

SWITCH$ = UCASE$(LTRIM$(RTRIM$(COMMAND$)))

'SWITCH$ = "/?" ' COMMENT OUT THIS LINE BEFORE COMPILING TO .EXE FILE!

IF INSTR(SWITCH$, "/?") THEN
  CLS
  PRINT "E2BOOT.EXE - a program for installing software in Motorola HC11E2 or similar"
  PRINT "microcontrollers which use internal 2K EEPROM as the sole program memory."
  PRINT "Source code for this version is E2BOOTNR.BAS."
  PRINT
  PRINT "Default serial port is COM1. To use COM2, type E2BOOT 2 at the DOS prompt."
  PRINT "Uses serial port on personal computer to connect to serial port on HC11-based"
  PRINT "equipment. No hardware handshake is needed, so short haul modems may be used"
  PRINT "over 1 or 2 twisted pairs (use direct RS-232 line if under about 500 feet long)."
  PRINT "This version is designed to be compatible with USB to serial converters"
  PRINT "and the DOS virtual machine in Windows XP."
  PRINT "This version is fixed at 1200 baud, no parity, 8 data & 1 stop."
  PRINT "The accompanying .BIN file sets the HC11 chip to these parameters also."
  PRINT
  PRINT "The HC11 EEPROM may be programmed either SOFT (2 milliseconds per EEPROM cell)"
  PRINT "or HARD (10 milliseconds per EEPROM cell)."
  PRINT "When RESET is pressed on the HC11 equipment, and if the HC11 is in boot mode,"
  PRINT "the programming process will start automatically."
  PRINT
  PRINT "Other files requred in default directory:"
  PRINT "    LATEST.S19 - Program to be downloaded, in Motorola S-record format."
  PRINT "    E2BOOT.BIN - Envoy program which goes to HC11 first, & arranges for"
  PRINT "    installation of user software.";
  END
END IF

IF INSTR(SWITCH$, "2") THEN
  commport$ = "2"
ELSE
  commport$ = "1"
END IF

FOR x = 1 TO 76
  READ OOPS$(x)
NEXT

bootprog$ = "e2boot.BIN"

LOADORNOT:

ON ERROR GOTO S19OOPS
OPEN "latest.s19" FOR INPUT AS #2
ON ERROR GOTO 0

'get program size from S-record file

MEMLOC = VAL("&HF800&")
psize = 0
DO
  LINE INPUT #2, s19lin$
  IF MID$(s19lin$, 2, 1) = "9" THEN EXIT DO
  trim$ = UCASE$(MID$(s19lin$, 9, LEN(s19lin$) - 10))
  psize! = psize! + LEN(trim$) \ 2
LOOP
SEEK #2, 1
overrun = 0
HSIZE$ = HEX$(psize)
topend! = MEMLOC + psize - 1
IF topend! > 65535 THEN overrun = topend! - 65535
IF overrun = 1 THEN sngpl$ = "byte"
IF overrun > 1 THEN sngpl$ = "bytes"

topend$ = HEX$(topend!)
PRINT
PRINT "Ready to install user software in HC11 EEPROM."
PRINT "If program has no vector list and is small enough, a default"
PRINT " pseudovector list will be added, targeted on upper end of RAM."
PRINT

PRINT "    File size: "; HSIZE$; " bytes hex ("; LTRIM$(STR$(psize)); " bytes decimal)"
IF overrun THEN
  PRINT "    Note: ";
  COLOR 14
  PRINT "not all of it will fit.";
  COLOR 7
  PRINT " The last";
  IF overrun > 1 THEN PRINT overrun;  ELSE PRINT " ";
  PRINT sngpl$; " will be discarded."
  PRINT "    That might be OK if the discarded "; sngpl$;
  IF sngpl$ = "bytes" THEN PRINT " are ";  ELSE PRINT " is ";
  PRINT "just part of a comment, etc.";
  PRINT "    Program will be trimmed to occupy EEPROM locations F800 to FFFF."
ELSE
  PRINT "    Program will occupy EEPROM locations F800 to "; topend$; "."
END IF
PRINT
PRINT "Actual EEPROM readout will be shown below as programming progresses."
PRINT
PRINT "Any bytes that did not program correctly will show highlighted in white"
PRINT "and blue like this: 8E 00 ";
COLOR 15, 1
PRINT "F7";
COLOR 7, 0
PRINT " CC C1 F8 etc."

' find out whether to do "hard" or "soft" EEPROM programming

PRINT
PRINT "Please place target system in bootload mode. CAUTION: if target is a secure"
PRINT "HC11, any reset in this mode will erase existing firmware."
PRINT
PRINT "When ready, press "; CHR$(34); "P"; CHR$(34); " for permanent hard-programming (takes longer)"
PRINT "Or press any other key to soft-program EEPROM (less EEPROM wear & tear)"
PRINT
PRINT "A soft-programmed EEPROM may begin to erase itself eventually."
PRINT "Chip should be hard-programmed before being put into actual service."
PRINT
PRINT "Press [ESC] if you do not want to proceed with the bootloading process."
PRINT
DO
  HS$ = UCASE$(INKEY$)
LOOP WHILE HS$ = ""
  IF HS$ = CHR$(27) THEN END

' ***** HC11 Reset Detector *****

' This block was added to get around a problem with a power-stealing modem
' used to communicate with an HC11. The break character sent by the HC11 would
' power down the modem and shut it off just when perfect response was needed.
' This block waits for reset, which it detects by the resulting framing error.
' It then begins normal download instantly, before the modem has time to lose
' its power due to the reset "break" signal removing the neg. supply voltage.
' The error handler "HC11RESET" sets HC11R variable to -1 to break the loop.
'
' A second advantage of this method is the elimination of 1 keypress. The
' download begins automatically as soon as the HC11 is reset.
'
' Advantage #3: No one needs to be at the PC to start the download.
'
' Advantage #4: Extra resets or a bouncy reset button do not cause a problem
' since each reset simply starts the downloading process again.
'
'
ON ERROR GOTO XMITOOPS
COM$ = "com" + commport$ + ":1200,N,8,1,BIN,CD0,CS0,DS0,OP0,RS"
P = VAL(MID$(port$, 4, 1))
OPEN COM$ FOR RANDOM AS #3 'LEN = 512
ECHO = -1
ON ERROR GOTO 0

FRERR:

PRINT "Press RESET button on HC11-based equipment, or [ESC] to return to editor."

HC11RESET = 0

ON ERROR GOTO HC11RESET

DO
  FramingError$ = INPUT$(LOC(3), #3) '   Framing error put to good use here
  IF INKEY$ = CHR$(27) THEN CLOSE : END
LOOP WHILE HC11RESET = 0 AND LEN(FramingError$) = 0

HC11RESET = 1
ON ERROR GOTO 0

' *******************************

' HC11 reset has been detected.

' Re-entry point to send loop if reset comes in during bootload

SEND:

COLUMN = 1
MEMLOC = VAL("&H0000&")
ASCII$ = ""
PRINT
COLOR 10, 2
PRINT "                ";
COLOR 7, 0
PRINT
COLOR 10, 2
PRINT " BOOTLOAD START ";
COLOR 7, 0
PRINT
COLOR 10, 2
PRINT "                ";
COLOR 7, 0
PRINT
PRINT
t! = TIMER
DO: LOOP WHILE t! > TIMER - 1

PRINT "Loading the programming system into the HC11... "
PRINT

SOUND 250, 1  'Acknowleges reset on internal speaker
SOUND 500, 1  'This is also handy to notify user of retries to the
SOUND 1000, 1 'HC11 bootloading process.

DISKOPS:

ON ERROR GOTO BINOOPS
OPEN bootprog$ FOR BINARY AS #1
BINDAT$ = ""

DO WHILE NOT EOF(1)
  CHAR$ = INPUT$(1, #1)
  BINDAT$ = BINDAT$ + CHAR$
LOOP

BOOTSIZE = LEN(BINDAT$)

SIZECHK:

CLOSE #1

IF LEN(BINDAT$) < 256 THEN
  bytes = LEN(BINDAT$)
  SHORTFALL = 256 - bytes
  BINDAT$ = BINDAT$ + STRING$(SHORTFALL, 1)
ELSEIF LEN(BINDAT$) > 256 THEN
  bytes = LEN(BINDAT$)
  overrun = bytes - 256
  BINDAT$ = LEFT$(BINDAT$, 256)
ELSE
END IF

ON ERROR GOTO XMITOOPS
P = VAL(MID$(port$, 4, 1))
ECHO = -1
COLUMN = 1

OOPS$ = "OK   "
delay! = TIMER
SLIN$ = CHR$(255) 'Signal the HC11 boot ROM program to check its baud rate.
PRINT #3, SLIN$;  'If 255 not received OK, HC11 will switch to 1200 baud.
                  'The HC11 boot ROM software does not echo this character.

sec$ = INPUT$(LOC(3), #3) 'Mop up HC11's "In security mode" signal if received
                          'Send the boot program (256 bytes).
FOR x = 1 TO 256          'Byte 0 already sent; send bytes 1 to 255
  SLIN$ = MID$(BINDAT$, x, 1)
  SENDBIN
NEXT x

ECHO = 0

SENT:

IF OOPS$ = "ERROR" THEN
  cleartime! = TIMER
  DO: LOOP WHILE TIMER < cleartime! + 2 'wait 2 sec. before clearing buffer
  PRINT
  PRINT
  PRINT "There was an error loading the programming system into the HC11."
  PRINT
  GOTO FRERR
END IF

CLOSE #1

DLDONE:

PRINT
PRINT "Done."
PRINT
PRINT STRING$(80, "-")
PRINT
MEMLOC = VAL("&HF800&")
IF HS$ = "P" THEN PTIME = 10 ELSE PTIME = 2
HSIZE$ = RIGHT$("000" + HSIZE$, 4)
SIZEHI$ = "&H" + LEFT$(HSIZE$, LEN(HSIZE$) - 2)
SIZELO$ = "&H" + RIGHT$(HSIZE$, 2)
SLIN$ = CHR$(VAL(SIZEHI$))' High byte of program size
SENDBIN
SLIN$ = CHR$(VAL(SIZELO$))' Low byte of program size
SENDBIN

' # of milliseconds to program each EEPROM byte
' (Value limited to 10 at receiving end)

SLIN$ = CHR$(PTIME)
SENDBIN

IF OOPS$ = "ERROR" THEN
  PRINT
  PRINT "There was an error sending commands to the programming system in the HC11."
  PRINT
  GOTO FRERR
END IF

BYTE = 0
ECHO = -1
PRINT "   COP enabled, security mode activated if available."
PRINT "   Installing user software:"
PRINT

'Convert file from S19 to binary and transmit to target equipment

SEEK #2, 1
BYTESTRING$ = ""
BINSIZE = 0
ECHO = -1

DO
  LINE INPUT #2, s19lin$
  IF MID$(s19lin$, 2, 1) = "9" THEN EXIT DO
  trim$ = UCASE$(MID$(s19lin$, 9, LEN(s19lin$) - 10))
 
  FOR x = 1 TO LEN(trim$) STEP 2
    BYTEVAL$ = "&H" + MID$(trim$, x, 2)
    BYTE = VAL(BYTEVAL$)
    SLIN$ = CHR$(BYTE)
    BINSIZE = BINSIZE + 1
    SENDBIN
    IF BINSIZE = 2048 THEN EXIT DO '2048 bytes available for user software
  NEXT x

LOOP

IF ASCII$ <> "" THEN CONVtoASC  'This call needed to display ASCII on last
                                'line of bytes if less than 16 bytes long.
ASCII$ = ""
CLOSE

IF OOPS$ = "ERROR" THEN
  PRINT
  PRINT "There was an error installing user software in the HC11 EEPROM."
  PRINT
  PRINT "Press [ESC] to retry, any other key to return to editor."
 
  DO
    KEYB$ = INKEY$
    IF KEYB$ = CHR$(27) THEN GOTO LOADORNOT
  LOOP WHILE KEYB$ = ""
 
  END

END IF

PRINT
PRINT "Downloading complete."; BINSIZE; "bytes installed."
PRINT "Program should be running in HC11."
QUIT

'If error encountered, print message and then exit to DOS

BINOOPS:

  COLOR 7, 0
  PRINT
 
  IF ERR = 53 OR ERR = 76 THEN
    PRINT
    PRINT "Cannot find ";
    IF ERR = 76 THEN PRINT "path to ";
    PRINT "binary source file "; bootprog$
    PRINT "Please check to see if it exists and that your typing is correct."
    QUIT
  ELSE
    PRINT
    PRINT "Cannot read file "; bootprog$; " due to error #"; LTRIM$(STR$(ERR)); " reported by DOS."
    PRINT "This is supposed to mean "; OOPS$(ERR); "."
    QUIT
  END IF

S19OOPS:

  COLOR 7, 0
  PRINT
 
  IF ERR = 53 OR ERR = 76 THEN
    PRINT
    PRINT "Cannot find ";
    IF ERR = 76 THEN PRINT "path to ";
    PRINT "S-record source file LATEST.S19"
    PRINT "Please check to see if it exists and that your spelling is correct."
    QUIT
  ELSE
    PRINT
    PRINT "Cannot read file LATEST.S19 due to error #"; LTRIM$(STR$(ERR)); " reported by DOS."
    PRINT "This is supposed to mean "; OOPS$(ERR); "."
    QUIT
  END IF

XMITOOPS:

  COLOR 7, 0
  PRINT
  PRINT OOPS$(ERR); ", unable to transmit data to HC11 properly."
 
  IF ERR = 68 THEN
    PRINT "DOS cannot locate a port called COM"; commport$; ":, perhaps because of a software or bus"
    PRINT "conflict. Rebooting the computer may fix the problem if COM"; commport$; ": actually exists."
  ELSEIF ERR = 24 THEN
    PRINT "There is no apparent connection to the HC11 or else it is not powered up."
    PRINT "The CTS,DSR or DCD signal is not being received from the HC11 interface."
    PRINT "Perhaps it is connected to the wrong serial port or a data switch is set wrong."
  ELSEIF ERR = 64 THEN
    PRINT "The "; CHR$(34); "file"; CHR$(34); " with the bad name is the communications port "; port$;
    PRINT "(incorrect port name, unavailable port or incorrect port parameters specified.)"
  END IF
 
  QUIT

RCVOOPS:
  
  COLOR 7, 0
  PRINT
 
  IF ERR = 57 THEN
    PRINT
    PRINT OOPS$(ERR); " or program in HC11 not communicating properly."
  ELSE
    PRINT OOPS$(ERR); ", can't receive data from HC11."
  END IF
 
  QUIT

HC11RESET:

  IF ERR = 57 THEN HC11RESET = -1: RESUME NEXT
  PRINT
  PRINT OOPS$(ERR)
  QUIT

RESET2:

  IF ERR = 57 THEN HC11RESET = -1: OOPS$ = "ERROR": RESUME SEND
  PRINT
  PRINT OOPS$(ERR)
  QUIT

'Error message table (dashes represent unused error numbers)

  DATA -,-,RETURN without GOSUB,Out of data,Illegal function call
  DATA Overflow,Out of memory,-,Subscript out of range,-,Division by zero
  DATA -,-,Out of string space,-,String formula too complex,-,-
  DATA No RESUME,RESUME without error,-,-,-,Device timeout,Device fault,-
  DATA Out of paper,-,-,-,-,-,-,-,-,-,-,-,CASE ELSE expected,Variable required
  DATA -,-,-,-,-,-,-,-,-,FIELD overflow,Internal error
  DATA Bad file name or number,File not found,Bad file mode,File already open
  DATA FIELD statement active,Device I/O error,File already exists
  DATA Bad record length,-,Disk full,Input past end of file,Bad record number
  DATA Bad file name,-,-,Too many files,Device unavailable
  DATA Communications buffer overflow,Permission denied,Disk not ready
  DATA Disk-media error,Advanced feature unavailable,Rename across disks
  DATA Path/file access error,Path not found

SUB CONVtoASC

       LOCATE , 59
      
       FOR x = 1 TO LEN(ASCII$)
         ASCHR$ = MID$(ASCII$, x, 1)
         CODE = ASC(ASCHR$)
        
         IF CODE <> 7 AND (CODE < 9 OR CODE > 13) AND (CODE < 28 OR CODE > 32) THEN
           PRINT ASCHR$;
         ELSE
           PRINT " ";
         END IF
      
       NEXT x
      
       PRINT
       COLUMN = 1
       ASCII$ = ""

END SUB

SUB ECHOBACK STATIC

'Receive echoed characters to confirm bootloader is functioning
'give up trying to communicate if no echo in 10 sec.
'this sub assumes that only 1 character will be echoed.

 start! = TIMER

 DO
  
   IF TIMER > start! + 29 THEN
     PRINT
     PRINT
     PRINT "Software handshake timeout - press any key"
     PRINT
     DO: LOOP WHILE INKEY$ = ""
     CLOSE : END
   END IF
  
   ON ERROR GOTO RESET2
  
   IF LOC(3) > 0 THEN
     BB$ = INPUT$(LOC(3), #3)
     ON ERROR GOTO 0
     IF SLIN$ <> BB$ THEN OOPS$ = "ERROR"
     IF ECHO THEN SCRNFORMAT
     EXIT SUB
   END IF

 LOOP
 END SUB

SUB QUIT
 
  PRINT
  PRINT "Press [ESC] to return to editor."
  DO: LOOP WHILE INKEY$ <> CHR$(27)
  CLOSE : END
  
END SUB

SUB SCRNFORMAT

     IF COLUMN = 1 THEN
       MLOC$ = RIGHT$("000" + HEX$(MEMLOC), 4)
       PRINT " "; MLOC$; ":  ";
       MEMLOC = MEMLOC + 16
     END IF
    
     H$ = HEX$(ASC(BB$))
     IF LEN(H$) = 1 THEN H$ = "0" + H$
     IF SLIN$ <> BB$ THEN COLOR 15, 1
     PRINT H$; " ";
     COLOR 7, 0
     IF COLUMN = 8 THEN PRINT " ";
     COLUMN = COLUMN + 1
     ASCII$ = ASCII$ + BB$
     IF COLUMN > 16 THEN CONVtoASC

END SUB

SUB SENDBIN STATIC

    ON ERROR GOTO XMITOOPS
    PRINT #3, SLIN$;
    ON ERROR GOTO 0
    ECHOBACK

END SUB

